# 11. Input devices

The topic of this week is "input devices". This week and next week topics (Output Devices) are very important for my final project. The class video is [here](https://vimeo.com/326957751).

### Assignments

- individual assignment:
      - measure something: add a sensor to a microcontroller board that you have designed and read it
- group assignment:
      - probe an input device's analog levels and digital signals

In the group assignment, we tested the analog levels and digital signals of sensors with using lab's oscilloscope. Pleae also see our [week 11 group assignment page](http://fabacademy.org/2019/labs/kamakura/assignments/week11/). 

I am get used to make some sensor devices with Arduino. Therefore, I tried to make the board which have two different sensors and write the program to read valuse of both two sensors (A/D converting in two different pins at the same time).

### 1st Challenge: Light and Accelemetor Sensor

I choose the following sensors to add on my original board.

- Light: PT15-21C/TR8　（Phototransistor)
      - [Digikey Link](https://www.digikey.com/product-detail/en/everlight-electronics-co-ltd/PT15-21C-TR8/1080-1380-1-ND/2676114)
- Accerometer sensors: ADXL343BCCZ
      - [Digikey Link](https://www.digikey.com/product-detail/en/ADXL343BCCZ/ADXL343BCCZ-ND)

#### Datasheet of ATTiny44.

I used ATTiny44 in this board. [The datasheet](http://academy.cba.mit.edu/classes/embedded_programming/doc8183.pdf) is helpful to check out pin configurations, which pins I can use for A/D convertion. 

The following figure show the pin configuration of ATTiny 44.

![](./week11_files/pinconfig-tiny44.png)

ATTiny 44 have seven pin configuration of ADC (PA0~PA7). Therefore, I could connect those pins to ADC out of each sensors. 

As learned from [week 9 (Embedded Programming)](../week09/), the direction of those pins should be configurated as "input" to read analog values from sensors. For example, if I want to read the value of the sensor that is connecte to PA7, it should be configured as follow.

in C language:

```
DDRA = (1<<PA7)
```


#### Designing PCB on EAGLE

First, designing PCB on Autodesk EAGLE. I will use the following parts.

- attiny44
- Phototransistor
- ADXL343BCCZ-ND
    - Footprint for EAGLE could be downloaded from [here](https://www.digikey.com/MyDigiKey/Redirect/AcceleratedDesigns?partNumber=ADXL343BCCZ-ND)
- 10K ohm register x 4
- 0ohm register x 2
- 1uF Capacitor x 2
- 20MHz Resonator x 2
- 2x3 Jumper Printed
- 6 pin FTDI Header

The schema is here:
![](./week11_files/1st-schema.png)

The board design is here:
![](./week11_files/1st-design-trace.png)

And, I will also design the outward form.

![](./week11_files/1st-design-outline.png)  

Export to PNG files.

<center>
  <img src="../week11_files/week11-1st-trace.png" width="45%">
  <img src="../week11_files/week11-1st-outline.png" width="45%">
</center>

Then, import to the mods and make CAM file for milling by Roland SRM-20.

![](./week11_files/1st-mods.png)

Before going to milling, I checked whether the patterns could be cut correctly.

![](./week11_files/1st-mods-preview.png)

OK, it's time to move for milling.

![](./week11_files/IMG_7374.jpg)

Then, it's completed to make a board. And, I will soldering components.
![](./week11_files/IMG_7377.jpg)

Soldering is completed.

![](./week11_files/IMG_7388.jpg)


#### Programming

I will use this FTDI connector. It could switch the supply voltage in 3.3V and 5V. I set 3.3V to connect my board and my PC.

![](./week11_files/FTDIModule.jpg)  


I wrote the following code in Arduino to check whether the phototransistor works or not.

```
#include <SoftwareSerial.h>

SoftwareSerial ss(0,1);
const int LIGHT_PIN = 7;

void setup() {
  // put your setup code here, to run once:
  ss.begin(9600);
  pinMode(LIGHT_PIN,INPUT);
  //analogReference();

   //
   // set clock divider to /1
   //
   CLKPR = (1 << CLKPCE);
   CLKPR = (0 << CLKPS3) | (0 << CLKPS2) | (0 << CLKPS1) | (0 << CLKPS0);

   ADCSRA = (1<<ADEN)
    | (1 << ADPS2) | ( 1 << ADPS1) | (1 << ADPS0);

   //TCCR0B |= (1<<CS01);

}

void loop() {
  // put your main code here, to run repeatedly:

  int val = analogRead(LIGHT_PIN);
  ss.println(val);

  delay(100);
}

```
The result, I checked with Serial monitor.

![](./week11_files/result1.png)

But the value of the light sensor always shows 1023....

According to the instructors, the problems of this board are...

- 3.3V regulation. 3.3v regulator should be connected only to accelemetor, and it is better to let microcontrollor work in 5.0v. However, my circuit designed whole in 3.3v. 
- I choosed different type of photo-transistor (light sensor), that could buy in local shop in Japan (not digikey). Therefore, the value of pull-up register would be different, and I should ckeck the datasheet for choosing appropriate value of register.
- Accelometer has SCL/SDA pins and those require I2C communications to read the values of sensor. I misconfigured to assigned the pins of the accelometer (those should be connected to SDA to PA7, and SCL to PA4).


### 2nd Challenge: Light and Hall Effect

I couldn't fix the problems of the accelemeter and lignt sensor board described above. Moreover, accelemetor chip (ADXL343) is quite difficult to soldering. Therefore, I changed to make different board that is added light and different sensor.

I choose hall effect sensor ([620-1403-ND](https://www.digikey.jp/product-detail/en/allegro-microsystems-llc/A1326LLHLT-T/620-1403-1-ND/2639995)).


#### Datasheet of ATTiny45

This 2nd challenge, I decided to follow to class sample boards. Therefore, I used ATTiny45. ATtiny45 is almostly the same with ATTiny44, however the number of pins in ATTiny45 are fewer than those of ATTiny44. 

As same with ATTiny44, I checked the datasheet of ATTiny45 to find out which pins are assigned as A/D converters.

![](./week11_files/datasheet-Tiny45.png)

According to the pin configuration of ATTiny45, PB2, PB3 and PB4 are assigned as ADC. Therefore, I could connect those pins to the sensors. 


  
#### Circuit Design.

In the 2nd challenge, I am combined the following two sample board (board for hall effect and the board for photo-transistor) provided in the class. 

<center>
<img src="http://academy.cba.mit.edu/classes/input_devices/mag/hello.mag.45.png" width="350">
<img src="http://academy.cba.mit.edu/classes/input_devices/light/hello.light.45.png" width="350">
</center>

In these sample boards, PB4 in ATTiny45 is connected to Hall Effect and PB3 is connected to photo-transistor. Here is the shecma of the 2nd board by Eagle.

![](./week11_files/eagle-schema-2nd.png)

And, here is the design pattern of the board.

![](./week11_files/eagle-degin-2nd.png)

Before moving the milling section, I checked whether my board works well or not with using Breadboard. And, I confirmed the sensor workd well.


<iframe src="https://player.vimeo.com/video/336063717" width="640" height="480" frameborder="0" allow="autoplay; fullscreen" allowfullscreen></iframe>

#### Milling and Soldering

After checking in breadboard, and also checking ERC/DRC, I exported the design of the board to PNG.

<img src="../week11_files/yosuke_input_dev_2nd_trace.png" width="340"> 
<img src="../week11_files/yosuke_input_dev_2nd_interor.png" width="340">

Then, making CAM file of the circuit in mods.

![](./week11_files/mods-trace.png)

Here is the view of the path for milling by Roland SRM-20. From the experience of [week 7 (Electronics Design) assignment](../week07/), I checked the path carefully before moving to milling part. That looks fine (no pathes that are not overlapped.)

![](./week11_files/rml-trace.png)

Also, making CAM file of the outline of the board.

![](./week11_files/mods-interor.png)

And, it's also corretctly made the path for milling.

![](./week11_files/rml-interor.png)


Then, milling PCB.

![](./week11_files/IMG_7458.jpg)

![](./week11_files/IMG_7460.jpg)

After milling the board, I solidered each parts. 

![](./week11_files/IMG_7463.jpg)

In this board, thise parts are required.


- ATTiny45 Microcontroller x 1
- 10k register x 2
- 0ohm register x 1 (for jumper)
- 1uF Capacitor x 1
- Hall Effect Sensor x 1
- Photo transistor x 1
- 2 x 3 pin for ISP connection
- 1 x 6 pin for FTDI connection

![](./week11_files/IMG_1804.jpg)

I took some mistake when soldering photo transistor again and again, so the copper circuit around phototransistor was broken. Then, I made a bypath near the photo transistor and solderied it.


#### Programming

It is required to show values of both hall effect and light at the same time. However, it is very easy to write such a program in Arduino. If we want to get the value of A/D converted data, we just define the pin number which is connected to the sensor, then write:

```
int val = analogRead(A3);

```


I just simply wrote the following program:

```

#include <SoftwareSerial.h>

const int lpin = A3;
const int magpin = A2;

SoftwareSerial ss(1,2);

void setup() {
  // put your setup code here, to run once:
  ss.begin(9600);
  

}

void loop() {
  // put your main code here, to run repeatedly:

  int val = analogRead(lpin);
  int val2 = analogRead(magpin);
  
  ss.print(val2);
  ss.print(',');
  ss.print(val);
  ss.print('\n');
  delay(500);
  
}

```

Then, uploaded the sketch to the board, and opened the "serial plotter", that show values of analogReaded data on the line chart. That works well.

![](./week11_files/serial-plotter-view.png)

<iframe src="https://player.vimeo.com/video/336131574" width="640" height="480" frameborder="0" allow="autoplay; fullscreen" allowfullscreen></iframe>


### Week 11 files

1st trial.

- Electronics Design
     - EAGLE Files ([sch](./week11_files/fabacademy_assignment_week11_1st.sch), [brd](./week11_files/fabacademy_assignment_week11_1st.brd))
     - PNG files ([trace](./week11_files/week11-1st-trace.png), [outline](./week11_files/week11-1st-outline.png))
     - CAM files by mods ([trace](./week11_files/week11-1st-trace.png.rml), [outline](./week11_files/week11-1st-outline.png.rml))
- Program
     - [Reading PhotoTransistor sensor value](./week11_files/fabacademy_week11_assignment_1st.ino)

2nd trial.

- Electronics Design
     - EAGLE Files ([sch](./week11_files/fabacademi_assignment_week11_2nd.sch), [brd](./week11_files/fabacademi_assignment_week11_2nd.brd))
     - PNG files ([trace](./week11_files/yosuke_input_dev_2nd_trace.png), [outline](week11_files/yosuke_input_dev_2nd_interor.png))
     - CAM files by mods  ([trace](./week11_files/yosuke_input_dev_2nd_trace.png.rml), [outline](./week11_files/yosuke_input_dev_2nd_interor.png.rml))
- Program
     - Reading Multiple Sensor Device code ([Arduino](./week11_files/fabacademy_assignment_week11_inputDevlices.ino)) 

